searchData={"items":[{"type":"module","title":"Experiment","doc":"","ref":"Experiment.html"},{"type":"macro","title":"Experiment.get_function/1","doc":"","ref":"Experiment.html#get_function/1"},{"type":"module","title":"Mozart.Data.ProcessModel","doc":"```\ndefstruct [:name, :tasks, :initial_task]\n```\n\nThis struct is used to create business process models. Once created, a process model\ncan be loaded into a `Mozart.ProcessEngine.html` and executed.\n\nAlso see Mozart.Task modules.\n\nExample:\n\n```\n%ProcessModel{\n      name: :user_task_process_model,\n      tasks: [\n        %User{\n          name: :user_task,\n          assigned_groups: [\"admin\"]\n        }\n      ],\n      initial_task: :user_task\n    }\n```","ref":"Mozart.Data.ProcessModel.html"},{"type":"module","title":"Mozart.Data.ProcessState","doc":"```\ndefstruct [\n    :uid,\n    :process_key,\n    :parent_pid,\n    :model_name,\n    :start_time,\n    :end_time,\n    :execute_duration,\n    open_tasks: %{},\n    completed_tasks: [],\n    data: %{},\n    complete: false\n  ]\nend\n```\n\nThis struct is used to represent the state of a `Mozart.ProcessEngine` execution.\n\nExample: (incrementally populated throughout process execution.)\n\n```\n%Mozart.Data.ProcessState{\n  uid: \"74146e68-088e-42b6-965e-20f4d7dbae16\",\n  process_key: \"74146e68-088e-42b6-965e-20f4d7123456\"\n  parent_pid: nil,\n  model_name: :call_external_service,\n  start_time: ~U[2024-05-27 15:22:01.847683Z],\n  end_time: ~U[2024-05-27 15:22:02.277273Z],\n  execute_duration: 429590,\n  open_tasks: %{},\n  completed_tasks: [\n    %{\n      data: %{},\n      function: #Function<0.23164178/1 in Mozart.ProcessModels.TestModels.call_exteral_services/0>,\n      name: :get_api_data,\n      type: :service,\n      next: nil,\n      __struct__: Mozart.Task.Script,\n      uid: \"e3326041-d203-46b2-8141-907f71421398\",\n      process_uid: \"74146e68-088e-42b6-965e-20f4d7dbae16\"\n    }\n  ],\n  data: %{\n    todo_data: %{\n      \"completed\" => false,\n      \"id\" => 1,\n      \"title\" => \"delectus aut autem\",\n      \"userId\" => 1\n    }\n  },\n  complete: true\n}\n```","ref":"Mozart.Data.ProcessState.html"},{"type":"module","title":"Mozart.Dsl.BpmProcess","doc":"","ref":"Mozart.Dsl.BpmProcess.html"},{"type":"macro","title":"Mozart.Dsl.BpmProcess.case_i/2","doc":"","ref":"Mozart.Dsl.BpmProcess.html#case_i/2"},{"type":"macro","title":"Mozart.Dsl.BpmProcess.case_task/2","doc":"","ref":"Mozart.Dsl.BpmProcess.html#case_task/2"},{"type":"macro","title":"Mozart.Dsl.BpmProcess.defprocess/2","doc":"","ref":"Mozart.Dsl.BpmProcess.html#defprocess/2"},{"type":"macro","title":"Mozart.Dsl.BpmProcess.insert_new_task/1","doc":"","ref":"Mozart.Dsl.BpmProcess.html#insert_new_task/1"},{"type":"function","title":"Mozart.Dsl.BpmProcess.parse_inputs/1","doc":"","ref":"Mozart.Dsl.BpmProcess.html#parse_inputs/1"},{"type":"function","title":"Mozart.Dsl.BpmProcess.parse_user_groups/1","doc":"","ref":"Mozart.Dsl.BpmProcess.html#parse_user_groups/1"},{"type":"macro","title":"Mozart.Dsl.BpmProcess.rule_task/2","doc":"","ref":"Mozart.Dsl.BpmProcess.html#rule_task/2"},{"type":"macro","title":"Mozart.Dsl.BpmProcess.script_task/2","doc":"","ref":"Mozart.Dsl.BpmProcess.html#script_task/2"},{"type":"macro","title":"Mozart.Dsl.BpmProcess.service_task/2","doc":"","ref":"Mozart.Dsl.BpmProcess.html#service_task/2"},{"type":"function","title":"Mozart.Dsl.BpmProcess.set_next_tasks/1","doc":"","ref":"Mozart.Dsl.BpmProcess.html#set_next_tasks/1"},{"type":"macro","title":"Mozart.Dsl.BpmProcess.subprocess_task/2","doc":"","ref":"Mozart.Dsl.BpmProcess.html#subprocess_task/2"},{"type":"macro","title":"Mozart.Dsl.BpmProcess.user_task/2","doc":"","ref":"Mozart.Dsl.BpmProcess.html#user_task/2"},{"type":"module","title":"Mozart.Event.TaskExit","doc":"","ref":"Mozart.Event.TaskExit.html"},{"type":"module","title":"Mozart.ProcessEngine","doc":"A ProcessEngine is dynamically spawned for the purpose of executing a `Mozart.Data.ProcessModel`.","ref":"Mozart.ProcessEngine.html"},{"type":"function","title":"Mozart.ProcessEngine.complete_on_task_exit_event/1","doc":"Complete subprocess due to a TaskExit event","ref":"Mozart.ProcessEngine.html#complete_on_task_exit_event/1"},{"type":"function","title":"Mozart.ProcessEngine.execute/1","doc":"Used to complete any \"complete-able\" open tasks. Task execution frequently spawns new\nopen tasks. Execute will continue to as long as there are \"complete-able\" open tasks.\n\nNote: Some types of task are complete-able immediately and some are not. For\nexample:\n  * A `Mozart.Task.Script` task is complete-able as soon as it is opened.\n  * A `Mozart.Task.User` task when a user completes the task.\n  * A `Mozart.Task.Receive` task is complete-able when a matching\n    `Mozart.Task.Send` task is received.","ref":"Mozart.ProcessEngine.html#execute/1"},{"type":"function","title":"Mozart.ProcessEngine.get_open_tasks/1","doc":"Gets the open tasks of the given process engine","ref":"Mozart.ProcessEngine.html#get_open_tasks/1"},{"type":"function","title":"Mozart.ProcessEngine.get_state/1","doc":"","ref":"Mozart.ProcessEngine.html#get_state/1"},{"type":"function","title":"Mozart.ProcessEngine.start_process/4","doc":"Use this function to create a ProcessEngine instance initialized with the\nname of the process model to be executed and any initialization data. The\nengine will start executing tasks with the execute/1 function is called.","ref":"Mozart.ProcessEngine.html#start_process/4"},{"type":"module","title":"Mozart.ProcessService","doc":"This modeule provides services required by individual `Mozart.ProcessEngine` instances. Currently,\nit has no user level functions. Subject to change.","ref":"Mozart.ProcessService.html"},{"type":"function","title":"Mozart.ProcessService.get_active_process_groups/4","doc":"","ref":"Mozart.ProcessService.html#get_active_process_groups/4"},{"type":"function","title":"Mozart.ProcessService.get_completed_process/1","doc":"Returns the state of the completed process corresponding to the process engine's uid","ref":"Mozart.ProcessService.html#get_completed_process/1"},{"type":"function","title":"Mozart.ProcessService.get_completed_process_db/0","doc":"Get process model db","ref":"Mozart.ProcessService.html#get_completed_process_db/0"},{"type":"function","title":"Mozart.ProcessService.get_completed_processes_local/1","doc":"","ref":"Mozart.ProcessService.html#get_completed_processes_local/1"},{"type":"function","title":"Mozart.ProcessService.get_process_model/1","doc":"Retrieves a process model by name.","ref":"Mozart.ProcessService.html#get_process_model/1"},{"type":"function","title":"Mozart.ProcessService.get_process_model_db/0","doc":"Get process model db","ref":"Mozart.ProcessService.html#get_process_model_db/0"},{"type":"function","title":"Mozart.ProcessService.get_process_models/0","doc":"","ref":"Mozart.ProcessService.html#get_process_models/0"},{"type":"function","title":"Mozart.ProcessService.get_process_pid_from_uid/1","doc":"Get a process pid from it UID","ref":"Mozart.ProcessService.html#get_process_pid_from_uid/1"},{"type":"function","title":"Mozart.ProcessService.get_processes_for_process_key/1","doc":"","ref":"Mozart.ProcessService.html#get_processes_for_process_key/1"},{"type":"function","title":"Mozart.ProcessService.get_state/0","doc":"","ref":"Mozart.ProcessService.html#get_state/0"},{"type":"function","title":"Mozart.ProcessService.get_user_task/1","doc":"Get a user task by uid","ref":"Mozart.ProcessService.html#get_user_task/1"},{"type":"function","title":"Mozart.ProcessService.get_user_task_db/0","doc":"Get process model db","ref":"Mozart.ProcessService.html#get_user_task_db/0"},{"type":"function","title":"Mozart.ProcessService.get_user_tasks_for_groups/1","doc":"Returns the user tasks that can be completed by users belonging to one of the input groups.","ref":"Mozart.ProcessService.html#get_user_tasks_for_groups/1"},{"type":"function","title":"Mozart.ProcessService.get_user_tasks_for_user/1","doc":"Get user tasks eligible for assignment and completion by the specified user.","ref":"Mozart.ProcessService.html#get_user_tasks_for_user/1"},{"type":"function","title":"Mozart.ProcessService.load_process_model/1","doc":"Loads a single process model in the repository.","ref":"Mozart.ProcessService.html#load_process_model/1"},{"type":"function","title":"Mozart.ProcessService.load_process_models/1","doc":"Loads a list of `Mozart.Data.ProcessModel`s into the state of the\nProcessService.","ref":"Mozart.ProcessService.html#load_process_models/1"},{"type":"function","title":"Mozart.ProcessService.update_for_completed_process/1","doc":"","ref":"Mozart.ProcessService.html#update_for_completed_process/1"},{"type":"module","title":"Mozart.Task.Case","doc":"Used to define a process model Case Task. Serves the same purpose as a BPMN2 Exclusive Gate.\nThe **cases** field is used to specify the multiple execution paths.\n\nExample:\n```\n%ProcessModel{\n      name: :case_process_model,\n      tasks: [\n        %Case{\n          name: :case_task,\n          cases: [\n            %{\n              expression: fn data -> data.value < 10 end,\n              next: :foo\n            },\n            %{\n              expression: fn data -> data.value >= 10 end,\n              next: :bar\n            }\n          ]\n        }\n      ],\n      initial_task: :case_task\n    }\n```","ref":"Mozart.Task.Case.html"},{"type":"module","title":"Mozart.Task.Join","doc":"Used to join parallel execution paths.\n\nExample:\n\n```\n%ProcessModel{\n      name: :parallel_process_model,\n      tasks: [\n        %Parallel{\n          name: :parallel_task,\n          multi_next: [:foo, :bar]\n        },\n        %Script{\n          name: :foo,\n          function: fn data -> Map.merge(data, %{foo: :foo}) end,\n          next: :join_task\n        },\n        %Script{\n          name: :bar,\n          function: fn data -> Map.merge(data, %{bar: :bar}) end,\n          next: :foo_bar\n        },\n        %Script{\n          name: :foo_bar,\n          function: fn data -> Map.merge(data, %{foo_bar: :foo_bar}) end,\n          next: :join_task\n        },\n        # Here's the Join task. Must receive all inputs before completing.\n        %Join{\n          name: :join_task,\n          inputs: [:foo, :foo_bar],\n          next: :final_service\n        },\n        %Script{\n          name: :final_service,\n          function: fn data -> Map.merge(data, %{final: :final}) end\n        }\n      ],\n      initial_task: :parallel_task\n    }\n```","ref":"Mozart.Task.Join.html"},{"type":"module","title":"Mozart.Task.Parallel","doc":"Use to create paralled execution paths.\n\nExample:\n\n```\n%ProcessModel{\n      name: :parallel_process_model,\n      tasks: [\n        # Here is the parallel task. Both :foo and :bar start paralled paths\n        %Parallel{\n          name: :parallel_task,\n          multi_next: [:foo, :bar]\n        },\n        %Script{\n          name: :foo,\n          function: fn data -> Map.merge(data, %{foo: :foo}) end,\n          next: :join_task\n        },\n        %Script{\n          name: :bar,\n          function: fn data -> Map.merge(data, %{bar: :bar}) end,\n          next: :foo_bar\n        },\n        %Script{\n          name: :foo_bar,\n          function: fn data -> Map.merge(data, %{foo_bar: :foo_bar}) end,\n          next: :join_task\n        },\n        %Join{\n          name: :join_task,\n          inputs: [:foo, :foo_bar],\n          next: :final_service\n        },\n        %Script{\n          name: :final_service,\n          function: fn data -> Map.merge(data, %{final: :final}) end\n        }\n      ],\n      initial_task: :parallel_task\n    }\n```","ref":"Mozart.Task.Parallel.html"},{"type":"module","title":"Mozart.Task.Receive","doc":"Used to model a task that waits for a PubSub message. Functional but needs\nsome work.\n\nExample:\n\n```\n%ProcessModel{\n      name: :process_with_receive_task,\n      tasks: [\n        %Receive{\n          name: :receive_task,\n          message_selector: fn msg ->\n            case msg do\n              :message -> %{message: true}\n              _ -> false\n            end\n          end\n        }\n      ],\n      initial_task: :receive_task\n    },\n    %ProcessModel{\n      name: :process_with_single_send_task,\n      tasks: [\n        %Send{\n          name: :send_task,\n          message: :message\n        },\n      ],\n      initial_task: :send_task\n    }\n```","ref":"Mozart.Task.Receive.html"},{"type":"module","title":"Mozart.Task.Rule","doc":"Used to model a Mozart run task. Called a Business Rule in BPMN2. Uses\nthe Elixir [rule_table](https://hex.pm/packages/rule_table) library.\n\nExample:\n\n```\n%ProcessModel{\n      name: :load_approval,\n      tasks: [\n        %Decision{\n          name: :loan_decision,\n          inputs: [:income],\n          rule_table:\n            Tablex.new(\"\"\"\n            F     income      || status\n            1     > 50000     || approved\n            2     <= 49999    || declined\n            \"\"\"),\n        },\n      ],\n      initial_task: :loan_decision\n    }\n```","ref":"Mozart.Task.Rule.html"},{"type":"module","title":"Mozart.Task.Script","doc":"Used to model a Service task. A service task calls a function and returns\ndata that is into the state data.\n\nExample:\n\n```\n%ProcessModel{\n      name: :process_with_single_service_task,\n      tasks: [\n        %Script{\n          name: :service_task,\n          inputs: [],\n          function: fn data -> Map.merge(data, %{single_service: true}) end\n        }\n      ],\n      initial_task: :service_task\n    }\n```","ref":"Mozart.Task.Script.html"},{"type":"function","title":"Mozart.Task.Script.__struct__/0","doc":"","ref":"Mozart.Task.Script.html#__struct__/0"},{"type":"module","title":"Mozart.Task.Send","doc":"Used to send a PubSub message to a waiting Receive task. Functioal but needs\nsome work.\n\nExample:\n\n```\n%ProcessModel{\n      name: :process_with_receive_task,\n      tasks: [\n        %Receive{\n          name: :receive_task,\n          message_selector: fn msg ->\n            case msg do\n              :message -> %{message: true}\n              _ -> false\n            end\n          end\n        }\n      ],\n      initial_task: :receive_task\n    },\n    %ProcessModel{\n      name: :process_with_single_send_task,\n      tasks: [\n        %Send{\n          name: :send_task,\n          message: :message\n        },\n      ],\n      initial_task: :send_task\n    }\n```","ref":"Mozart.Task.Send.html"},{"type":"module","title":"Mozart.Task.Service","doc":"Used to model a Service task. A service task calls a function and returns\ndata that is into the state data.\n\nExample:\n\n```\n%ProcessModel{\n      name: :process_with_single_service_task,\n      tasks: [\n        %Script{\n          name: :service_task,\n          inputs: [],\n          function: fn data -> Map.merge(data, %{single_service: true}) end\n        }\n      ],\n      initial_task: :service_task\n    }\n```","ref":"Mozart.Task.Service.html"},{"type":"function","title":"Mozart.Task.Service.__struct__/0","doc":"","ref":"Mozart.Task.Service.html#__struct__/0"},{"type":"module","title":"Mozart.Task.Subprocess","doc":"Used to model a task that is completed by executing and completing a\nsubprocess. Called a **call activity** in BPMN2.\n\nExample:\n\n```\n%ProcessModel{\n      name: :call_process_model,\n      tasks: [\n        %Subprocess{\n          name: :call_process_task,\n          sub_process_model_name: :service_subprocess_model,\n          next: :service_task1\n        },\n        %Script{\n          name: :service_task1,\n          function: fn data -> Map.put(data, :value, data.value + 1) end\n        }\n      ],\n      initial_task: :call_process_task\n    },\n    %ProcessModel{\n      name: :service_subprocess_model,\n      tasks: [\n        %Script{\n          name: :service_task,\n          function: fn data -> Map.put(data, :subprocess_data, \"subprocess data\") end\n        }\n      ],\n      initial_task: :service_task\n    }\n```","ref":"Mozart.Task.Subprocess.html"},{"type":"module","title":"Mozart.Task.Timer","doc":"Used to model a task waits a specified duration and automatically completes.\n\nExample:\n\n```\n%ProcessModel{\n    name: :call_timer_task,\n    tasks: [\n      %Timer{\n        name: :wait_1_seconds,\n        timer_duration: 10\n      },\n    ],\n    initial_task: :wait_1_seconds\n  }\n```","ref":"Mozart.Task.Timer.html"},{"type":"module","title":"Mozart.Task.User","doc":"Used to model a task that must be completed by a system user.\n\nExample:\n\n```\n%ProcessModel{\n      name: :user_task_process_model,\n      tasks: [\n        %User{\n          name: :user_task,\n          assigned_groups: [\"admin\"]\n        }\n      ],\n      initial_task: :user_task\n    }\n```","ref":"Mozart.Task.User.html"},{"type":"module","title":"Mozart.UserService","doc":"Provides client applications with functionality related to users and groups. Much more\nfunctionality to come.","ref":"Mozart.UserService.html"},{"type":"function","title":"Mozart.UserService.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Mozart.UserService.html#child_spec/1"},{"type":"function","title":"Mozart.UserService.get_assigned_groups/1","doc":"Returns the groups that a user is a member of.","ref":"Mozart.UserService.html#get_assigned_groups/1"},{"type":"function","title":"Mozart.UserService.get_user/1","doc":"Given a user id, return detailed user info.","ref":"Mozart.UserService.html#get_user/1"},{"type":"function","title":"Mozart.UserService.insert_user/1","doc":"Adds a new user to the system.","ref":"Mozart.UserService.html#insert_user/1"},{"type":"module","title":"SampleFunctions","doc":"","ref":"SampleFunctions.html"},{"type":"function","title":"SampleFunctions.square/1","doc":"","ref":"SampleFunctions.html#square/1"},{"type":"extras","title":"Introduction to BPM","doc":"# Introduction to BPM\n\nThe next several sections provide an introduction to to ***Business Process Management***.","ref":"intro_bpm.html"},{"type":"extras","title":"What is a Business Process? - Introduction to BPM","doc":"Companies large and small have business processes that they regularly execute. Let's just call them \"processes\" for brevity's sake. Insurance companies have processes to process a claim. Banks have processes to approve loans. The government have processes to award contracts. \n\nBusiness processes have the following charateristics:\n\n* A business process is comprised of some number of individual tasks. \n* Tasks are performed in an order prescribed by network of task connections.\n* Processes can branch to multiple alternative sets of tasks depending on the state of the process.\n* Processes can branch to multiple parallel paths as well.\n* There are multiple kinds of tasks. Some tasks are performed by users. Some tasks are performed by calling services. There are many other kinds of tasks as well.\n* Processes store data to be used by tasks to perform their functions.\n* Processes allow tasks to add data for use by downstream tasks.","ref":"intro_bpm.html#what-is-a-business-process"},{"type":"extras","title":"What is Business Process Management (BPM)? - Introduction to BPM","doc":"Business Process Management (BPM) is the organized and directed activity of a company to control, monitor and improve the quality of their business processes. They do this with the aid of a BPM platform made available by dozens of commercial vendors.\n\nBPM platforms provide the following functions:\n\n* Provide developers with the ability of define and store process models.\n* Provide developers with the ability have certain tasks completed by making service calls.\n* Provide users and administrators with the ability to start process instances.\n* Provide users with the ability to assign user tasks to specified users.\n* Prvide users with the ability to assign user tasks to members of a specified user group.\n\nThe goals of Business Process Management are:\n\n* Better Process Reliablibility\n* Faster Process Execugtion\n* Reduced Training Requirements\n* Incremental Process Improvement\n* Provide Process Metrics\n* Cost Savings","ref":"intro_bpm.html#what-is-business-process-management-bpm"},{"type":"extras","title":"When is BPM Applicable - Introduction to BPM","doc":"This is difficult to answer exactly, but BPM may be applicable when:\n\n* Processes are performed on a regular basis.\n* Successful process execution is crucial to the business.\n* Current processes are error prone.\n* The speed of process completion is very important.\n* Processes are well defined.\n* Multiple business groups and/or people participate in process execution.","ref":"intro_bpm.html#when-is-bpm-applicable"},{"type":"extras","title":"Introduction to Mozart","doc":"# Introduction to Mozart","ref":"app_overview.html"},{"type":"extras","title":"Process Models & Engines - Introduction to Mozart","doc":"One of the primary concepts of *Mozart* is the **Process Model** implemented by `Mozart.Data.ProcessModel`. A *process model* is an executable representation of a business process. \n\nIf you are a developer working for a bank, you might be asked to develop a process model for handling a customer loan application.\n\nBut what does it mean for a process model to be executable? That's where another major concept comes in - the **Process Engine** implemented by `Mozart.ProcessEngine`. A process model and some initial data are loaded into a process engine, after which process execution is initiated.","ref":"app_overview.html#process-models-engines"},{"type":"extras","title":"Process Model Tasks - Introduction to Mozart","doc":"What happens when a process engine executes a process model? That brings us to the notion of **Tasks**. Tasks are defined in the process model. Each task represents some work that needs to be completed. The job of the process engine is to drive completion of the tasks specified in the process model. You will hear the terminology of **task completion** frequently. To **complete** a task means to perform the work required of the task.\n\nMozart provides arounds a dozen different type of *tasks*. The two most ofen used are the `Mozart.Task.User` task and the `Mozart.Task.Script` task. \n\nThe task types currently supported are:\n\n| Task Type               |  Description |\n|-----|-----|\n| User Task               | Performed by a user |\n| Service Task            | Performed by calling a service. |\n| Subprocess Task         | Performed by calling a subprocess. |\n| Timer Task              | Waits for expiration of a timer. |\n| Receive Task            | Waits for a subscribed PubSub event. |\n| Send Task               | Sends a PubSub event. |\n| Rule Task               | Performed by evaluating a set of rules represented in a table. |\n| Case Task             | Selects one of many process paths. |\n| Parallel Task           | Initiates two or more parallel process paths. |\n| Join Task               | Sychronizes on completion of two or more process paths. |","ref":"app_overview.html#process-model-tasks"},{"type":"extras","title":"Process Service - Introduction to Mozart","doc":"(Needs an update for more process services functions) When a process model has been developed, it is stored in a process model repository for later use. The repository is implemented by `Mozart.ProcessService`. When system users are ready to execute a process model, they retrieve it from the repository and use it to start a process engine execution.","ref":"app_overview.html#process-service"},{"type":"extras","title":"Processes Model Execution - Introduction to Mozart","doc":"Earlier we said that the job of the process engine is to drive a process model execution to completion. Let's discuss what that means.","ref":"app_overview.html#processes-model-execution"},{"type":"extras","title":"Completing Tasks - Introduction to Mozart","doc":"Every process model must specify an **initial task**. This will be the first task opened for completion. Completion of the initial task will often, though not always, trigger the opening of additional tasks. Every task that completes will potentially trigger the opening of additional tasks. At least one task in a process model will not trigger any downstream tasks. Eventually, a process model will stop generating new open tasks and it's process engine will terminate.\n\nSo, completing the execution of a process model equates to repeatedly opening and completing tasks until the process as a whole is complete.","ref":"app_overview.html#completing-tasks"},{"type":"extras","title":"Accumulating Data - Introduction to Mozart","doc":"Almost every task that is opened will need some data to complete. Further, when a task completes, it will introduce some new data for downstream tasks to use. Thus, to a great extent, completing a processes equates to generating a store of information.\n\nAll of the data generated by task completion is stored in the **data** property of `Mozart.ProcessEngine`.","ref":"app_overview.html#accumulating-data"},{"type":"extras","title":"Producing Side Effects - Introduction to Mozart","doc":"To be useful, almost all process model executions will produce some side effects. Take the process of processing a bank loan application for example. What are the possible side effects?\n\n* Sending a notice that the loan is approved or declined.\n* Depositing money in the customers account.\n* Setting up the loan in the banks backend system.","ref":"app_overview.html#producing-side-effects"},{"type":"extras","title":"Conclusion - Introduction to Mozart","doc":"Hopefully, the mechanics of Business Process Managment is now much clearer. In the newxt section, we will use Mozart to execute business processes.","ref":"app_overview.html#conclusion"},{"type":"extras","title":"First Process Execution","doc":"# First Process Execution\n\nNow we will put together what we've lerned so far to execute a process model.","ref":"first_process_execution.html"},{"type":"extras","title":"To Follow Along.. - First Process Execution","doc":"To follow along create a new Elxir project and then add Mozart as a dependency. Your dependency should look like this:\n\n```elixir\n  defp deps do\n    [\n      {:mozart, \"~> 0.1\"}\n    ]\n  end\n```","ref":"first_process_execution.html#to-follow-along"},{"type":"extras","title":"Construct the Process Model - First Process Execution","doc":"First, let us construct a simple process model that will have a single service task. The service task will simply add one to a data property. Here is the process model:\n\n```elixir\n    %ProcessModel{\n        name: :process_with_single_service_task,\n        tasks: [\n          %Script{\n            name: :service_task,\n            inputs: [:x],\n            function: fn data -> Map.put(data, :x, data.x + 1) end\n          }\n        ],\n        initial_task: :service_task\n    }\n```\n\nThe process model definition is pretty simple. It uses two Mozart structures: `Mozart.Task.Script` and `Mozart.Data.ProcessModel`. The name of the process model is **:process_with_single_service_task**. It has just one service task which is named **:service_task**, and this task is the **initial_task** that will be opened upon upon process execution. The service task does not specify a next task so the process will complete after this task is completed.","ref":"first_process_execution.html#construct-the-process-model"},{"type":"extras","title":"Load the Process Model into repository - First Process Execution","doc":"Now let's load this process model into the process model repository, which is implemented by (`Mozart.ProcessService`). First, let's paste a few aliases into our iex session:\n\n```elixir\nalias Mozart.ProcessEngine, as: PE\nalias Mozart.ProcessService, as: PS\nalias Mozart.Task.Script\nalias Mozart.Data.ProcessModel\n\n```\n\nNow we are ready to load our process model into the repository. Copy the following into your iex session:\n\n```elixir\nprocess_model = \n    %ProcessModel{\n        name: :process_with_single_service_task,\n        tasks: [\n          %Script{\n            name: :service_task,\n            inputs: [:x],\n            function: fn data -> Map.put(data, :x, data.x + 1) end\n          }\n        ],\n        initial_task: :service_task\n    }\nPS.load_process_model(process_model)\nPS.get_process_model(:process_with_single_service_task)\n\n```\n\nFirst, we assigned a process model to a variable.\nSecondly, we loaded the process definition into our repository.\nFinally, we pulled the process model from the repository as verification. At the very bottom of the output, you should see the result of calling **get_process_model(:process_with_single_service_task)**:\n\n\n```\niex [12:52 :: 8] > PS.get_process_model(:process_with_single_service_task)\n%Mozart.Data.ProcessModel{\n  name: :process_with_single_service_task,\n  tasks: [\n    %Mozart.Task.Script{\n      name: :service_task,\n      function: #Function<42.105768164/1 in :erl_eval.expr/6>,\n      next: nil,\n      uid: nil,\n      data: %{},\n      type: :service\n    }\n  ],\n  initial_task: :service_task\n}\n```","ref":"first_process_execution.html#load-the-process-model-into-repository"},{"type":"extras","title":"Start a Process Engine and Execute the Process Model - First Process Execution","doc":"Now let us start a process engine, initializing it with our process model and some data. Then we call the function that will invoke process execution. Copy the following into your iex session:\n\n```elixir\n{:ok, pid, uid, _process_key} = \n  PE.start_process(:process_with_single_service_task, %{x: 0})\nPE.execute(pid)\n\n```\n\nFirst, we started a process using `Mozart.ProcessEngin.start_process/2`. We passed it the name of the process model to be executed and some initial data required for process execution. The function returns a tuple with three values: \n\n* The **pid** of the resulting process engine.\n* The **uid** of the process instance. Every process execution will need a unique identifier. You will see how this is used going forward.\n* Finally, a **process_key** is returned. Process modes are hierarchial, meaning they can be composed of multiple, nested subprocesses. The top level business process and all of its subprocesses will share the same *process_key*.\n\nAfter creating the process engine instance, we then call `Mozart.ProcessEngine.execute/1` which causes the process engine to start completing any tasks that are ready to be completed. After calling the **execute** function, you should see something like:\n\n```\niex [12:22 :: 12] > {:ok, pid, uid, _process_key} = PE.start_process(:process_with_single_service_task, %{x: 0})\n12:27:40.898 [info] Start process instance [process_with_single_service_task][0800de9a-8ec5-4906-bf50-bd09321f5982]\n{:ok, #PID<0.277.0>, \"0800de9a-8ec5-4906-bf50-bd09321f5982\", \"0800de9a-8ec5-4906-bf50-bd09321f1234\"}\niex [12:22 :: 13] > PE.execute(pid)\n:ok\n12:27:40.899 [info] New task instance [service_task][da19b03a-009a-42aa-948c-4ad139d0fe66]\n12:27:40.899 [info] Complete service task [service_task[da19b03a-009a-42aa-948c-4ad139d0fe66]\n12:27:40.899 [info] Process complete [process_with_single_service_task][0800de9a-8ec5-4906-bf50-bd09321f5982]\n```\n\nThe logs show that:\n\n1. A new process engine was created.\n1. A new service task was opened.\n1. The service task completed.\n1. And finally the process engine completed executing the process model.","ref":"first_process_execution.html#start-a-process-engine-and-execute-the-process-model"},{"type":"extras","title":"Verify the Results - First Process Execution","doc":"Let's verify the results of our process model execution. Copy the following into your iex session:\n\n```elixir\nPS.get_completed_process(uid)\n\n```\n\nand you should see:\n\n```elixir\niex [12:22 :: 14] > PS.get_completed_process(uid)\n%Mozart.Data.ProcessState{\n  uid: \"0800de9a-8ec5-4906-bf50-bd09321f5982\",\n  parent_pid: nil,\n  model_name: :process_with_single_service_task,\n  start_time: ~U[2024-05-28 17:27:40.898328Z],\n  end_time: ~U[2024-05-28 17:27:40.899505Z],\n  execute_duration: 1177,\n  open_tasks: %{},\n  completed_tasks: [\n    %{\n      data: %{},\n      function: #Function<42.105768164/1 in :erl_eval.expr/6>,\n      name: :service_task,\n      type: :service,\n      next: nil,\n      __struct__: Mozart.Task.Script,\n      uid: \"da19b03a-009a-42aa-948c-4ad139d0fe66\",\n      process_uid: \"0800de9a-8ec5-4906-bf50-bd09321f5982\"\n    }\n  ],\n  data: %{x: 1},\n  complete: true\n}\n```\n\nWe see the start and end times of process execution as well as the execution duration of 1177 microseconds (or 1.177 milliseconds). We see that tasks that were completed. And, finally, we see that the service task has done its job of adding 1 to the value of **x**.","ref":"first_process_execution.html#verify-the-results"},{"type":"extras","title":"Mozart Task Types","doc":"# Mozart Task Types","ref":"task_types.html"},{"type":"extras","title":"Properties Common to All Task Types - Mozart Task Types","doc":"All of the task types have the following properties:\n* **name**: an atome specifying the name of the task. Must be unique within the scope of the parent process model.\n* **next**: the name of the task that should be opened after completion of the current task. Exception: The task type `Mozart.Task.Paralled` doesn't have a next field. Instead, it has a multi_next field, which specifies next tasks that should be opened in parallel.\n* **uid**: When a task is opened (instantiated) it is assigned a unique identifier, i.e. its *uid*.\n* **type**: Specifies the type of the task. This value is defaulted automatically for each task type.","ref":"task_types.html#properties-common-to-all-task-types"},{"type":"extras","title":"Service Task - Mozart Task Types","doc":"A **Service Task** (`Mozart.Task.Script`) performs its work by calling an Elixir function. This function could perform a computation, call an external JSON service, retrieve data from a database, etc.\n\nA service task has two unique fields: **:function** and **:inputs**.\n\n* The **:function** field specifies the function that the service task should apply for the purpose of returning output data into the process state.\n* The **:inputs** field is used to select which process data fields are passed to the task's function. If no value is supplied for this field, the entire process data is passed.","ref":"task_types.html#service-task"},{"type":"extras","title":"Service Task example - Mozart Task Types","doc":"If you are following along, open an Elixir project that has Mozart as a dependency.\n\n```\niex -S mix\n\n```\n\nNow paste the following alias' into your iex session\n\n```\n alias Mozart.Data.ProcessModel\n alias Mozart.Task.Script\n alias Mozart.ProcessEngine, as: PE\n alias Mozart.ProcessService, as: PS\n\n```\n\nNow define a process model with a single service task and assign it to a variable:\n\n```\nmodel = %ProcessModel{\n    name: :process_with_single_service_task,\n    tasks: [\n      %Script{\n        name: :service_task,\n        inputs: [:x, :y],\n        function: fn data -> Map.put(data, :sum, data.x + data.y) end\n      }\n    ],\n    initial_task: :service_task\n}\n\n```\n\nWe have specified that state data properties **:x** and **:y** should be available to the task's function. \n\nThe task function will take the sum of **x** and **y** and assign the result to a new the new property **sum**.\n\nNow paste the following into your iex session to execute your process model:\n\n```\nPS.load_process_model(model)\ndata = %{x: 1, y: 1}\n{:ok, ppid, uid, _process_key} = PE.start_process(:process_with_single_service_task, data)\nPE.execute(ppid)\n\n```\n\nNow, to view the state of the completed process, paste the following into your iex session:\n\n```\nPS.get_completed_process(uid)\n\n```\n\nand should see something like this:\n\n```\niex [12:21 :: 11] > PS.get_completed_process(uid)\n%Mozart.Data.ProcessState{\n  uid: \"53d0220b-4e0c-42bb-9154-7e4aeff83837\",\n  parent_pid: nil,\n  model_name: :process_with_single_service_task,\n  start_time: ~U[2024-05-30 17:22:49.156516Z],\n  end_time: ~U[2024-05-30 17:22:49.161447Z],\n  execute_duration: 4931,\n  open_tasks: %{},\n  completed_tasks: [\n    %{\n      function: #Function<42.105768164/1 in :erl_eval.expr/6>,\n      name: :service_task,\n      type: :service,\n      next: nil,\n      __struct__: Mozart.Task.Script,\n      uid: \"443319db-9cef-47c0-9366-9b95ec1fa42b\",\n      inputs: [:x, :y],\n      process_uid: \"53d0220b-4e0c-42bb-9154-7e4aeff83837\"\n    }\n  ],\n  data: %{sum: 2, y: 1, x: 1},\n  complete: true\n}\n```\n\nWe see that the new property **sum** has been added to the state of the completed process.","ref":"task_types.html#service-task-example"},{"type":"extras","title":"User Task - Mozart Task Types","doc":"A **User Task** (`Mozart.Task.User`) works by having a user complete some required task. This often takes the form of a user using a GUI to examine a subset of process data and then supplying additional data. \n\n**Important Note**: Users interact with a BPM platfrom such as Mozart by way of user application of some kind. The user application will allow the user to find tasks which may be assigned to him. Once the user accepts responsibility for a task, the application will then provide a user interface appropriate for accomplishing the given task. \n\nA user task has two unique fields: **:assigned_groups** and **:inputs**.\n\n* The **:assigned_groups** field specifies the user groups that are elibigle to complete the task. \n* The **:inputs** field is used to select which process data fields are passed to the user that will compleete the task. If no value is supplied for this field, the entire process data is passed.","ref":"task_types.html#user-task"},{"type":"extras","title":"User Task example - Mozart Task Types","doc":"If you are following along, open an Elixir project that has Mozart as a dependency.\n\n```\niex -S mix\n\n```\n\nNow paste the following alias' into your iex session\n\n```\n alias Mozart.Data.ProcessModel\n alias Mozart.Task.User\n alias Mozart.ProcessEngine, as: PE\n alias Mozart.ProcessService, as: PS\n\n```\n\nNow we define a process model with a single service task and assign it to a variable:\n\n```\nmodel = %ProcessModel{\n      name: :user_task_process_model,\n      tasks: [\n        %User{\n          name: :user_task,\n          inputs: [:x, :y],\n          assigned_groups: [\"admin\"]\n        }\n      ],\n      initial_task: :user_task\n  }\n\n```\n\nWe have specified that state data properties **:x** and **:y** should be available to the user that performs the task.\n\nThe user will compute the sum of **x** and **y** and assign the result to a new the new property **sum**.\n\nNow paste the following into your iex session to execute your process model:\n\n```\nPS.clear_user_tasks()\nPS.load_process_model(model)\ndata = %{x: 1, y: 1}\n{:ok, ppid, uid, _process_key} = PE.start_process(:user_task_process_model, data)\nPE.execute(ppid)\n\n```\n\nAt this point, a user task has been opened and is available for a user to claim and complete. Now we need to find a user task that can be claimed by persons in specified groups. We can do that like this:\n\n```\n[user_task] = PS.get_user_tasks_for_groups([\"admin\"])\n\n```\n\nwhich gives this result:\n\n```\n13:29:25.959 [info] New task instance [user_task][09fe2c87-d2c8-401b-a4df-83a7edab987d]\niex [13:28 :: 11] > PS.get_user_tasks_for_groups([\"admin\"])\n[\n  %{\n    complete: false,\n    data: %{y: 1, x: 1},\n    function: nil,\n    name: :user_task,\n    type: :user,\n    next: nil,\n    __struct__: Mozart.Task.User,\n    uid: \"09fe2c87-d2c8-401b-a4df-83a7edab987d\",\n    assigned_groups: [\"admin\"],\n    inputs: [:x, :y],\n    process_uid: \"69b51421-cc10-45d7-8ca2-70c0e232cb82\"\n  }\n]\n```\n\nNow, pulling the task uid from the results above, we can complete the user task like this:\n\n```\nPE.complete_user_task(ppid,  user_task.uid, %{sum: 2})\n\n```\n\nwhich will produce a result like this:\n\n```\niex [16:54 :: 12] > PE.complete_user_task(ppid,  user_task.uid, %{sum: 2})\n16:56:11.417 [info] Complete user task [user_task][9b88be91-3b99-4e9c-b362-0f0dc5fb604c]\n16:56:11.417 [info] Process complete [user_task_process_model][d7bed372-94c4-477a-be0a-dcc7417c6e60]\n** (exit) exited in: GenServer.call(#PID<0.273.0>, {:complete_user_task, \"9b88be91-3b99-4e9c-b362-0f0dc5fb604c\", %{sum: 2}}, 5000)\n    ** (EXIT) shutdown\n    (elixir 1.16.2) lib/gen_server.ex:1114: GenServer.call/3\n    iex:12: (file)\n```\n\nDon't be concerned regarding the **shutdown** notice. It is normal. It simply means tht the process engine shutdown becuase the process had finished.\n\nNow we can verify that the process has completed and shows the expected value for process data:\n\n```\nPS.get_completed_process(uid)\n\n```\n\nand should see something like this:\n\n```\niex [13:28 :: 12] > PS.get_completed_process(uid)\n%Mozart.Data.ProcessState{\n  uid: \"69b51421-cc10-45d7-8ca2-70c0e232cb82\",\n  parent_pid: nil,\n  model_name: :user_task_process_model,\n  start_time: ~U[2024-05-30 18:29:25.954801Z],\n  end_time: ~U[2024-05-30 21:35:58.017106Z],\n  execute_duration: 11192062305,\n  open_tasks: %{},\n  completed_tasks: [\n    %{\n      complete: false,\n      function: nil,\n      name: :user_task,\n      type: :user,\n      next: nil,\n      __struct__: Mozart.Task.User,\n      uid: \"09fe2c87-d2c8-401b-a4df-83a7edab987d\",\n      assigned_groups: [\"admin\"],\n      inputs: [:x, :y],\n      process_uid: \"69b51421-cc10-45d7-8ca2-70c0e232cb82\"\n    }\n  ],\n  data: %{sum: 3, y: 1, x: 1},\n  complete: true\n}\n```\n\nWe see that the new property **sum** has been added to the state of the completed process.","ref":"task_types.html#user-task-example"},{"type":"extras","title":"Rule Task - Mozart Task Types","doc":"A **Rule Task** (`Mozart.Task.Rule`) performs its work by evaluating a rule table using process data. \n\nRule tasks use [Tablex](https://hexdocs.pm/tablex/0.3.1/readme.html), an incredibly useful Elixir library for evaluating [decision tables](https://en.wikipedia.org/wiki/Decision_table).\n\nA rule task has two unique fields: **:decision_args** and **:rule_table**.\n\n* The **:inputs** field specifies the state data fields used to evaluate the rule table.\n* The **:rule_table** field holds the Tablex table definition.","ref":"task_types.html#rule-task"},{"type":"extras","title":"Rule Task example - Mozart Task Types","doc":"If you are following along, open an Elixir project that has Mozart as a dependency.\n\n```\niex -S mix\n\n```\n\nNow paste the following alias' into your iex session\n\n```\n alias Mozart.Data.ProcessModel\n alias Mozart.Task.Rule\n alias Mozart.ProcessEngine, as: PE\n alias Mozart.ProcessService, as: PS\n\n```\n\nNow define a process model with a single service task and assign it to a variable:\n\n```\nmodel = %ProcessModel{\n    name: :loan_approval,\n    tasks: [\n      %Rule{\n        name: :loan_decision,\n        inputs: [:income],\n        rule_table:\n          Tablex.new(\"\"\"\n          F     income      || status\n          1     > 50000     || approved\n          2     <= 49999    || declined\n          \"\"\")\n      }\n    ],\n    initial_task: :loan_decision\n}\n\n```\n\nWe have specified that state data property **:income** should be available as an input field to the rule table.\n\nThe **:rule_table** property holds the Tablex table to be evaluated.\n\nNow paste the following into your iex session to execute your process model:\n\n```\nPS.load_process_model(model)\ndata = %{income: 3000}\n{:ok, ppid, uid, _process_key} = PE.start_process(:loan_approval, data)\nPE.execute(ppid)\n\n```\n\nNow, to view the state of the completed process, paste the following into your iex session:\n\n```\nPS.get_completed_process(uid)\n\n```\n\nand should see something like this:\n\n```\niex [20:45 :: 11] > PS.get_completed_process(uid)\n%Mozart.Data.ProcessState{\n  uid: \"b79fe224-52b9-4457-b885-1234c47c0eb3\",\n  parent_pid: nil,\n  model_name: :loan_approval,\n  start_time: ~U[2024-06-01 01:45:57.687401Z],\n  end_time: ~U[2024-06-01 01:45:57.695101Z],\n  execute_duration: 7700,\n  open_tasks: %{},\n  completed_tasks: [\n    %{\n      name: :loan_decision,\n      type: :rule,\n      next: nil,\n      __struct__: Mozart.Task.Rule,\n      uid: \"a045ccfe-21d1-410f-aa87-6e020017e48d\",\n      inputs: [:income],\n      rule_table: %Tablex.Table{\n        hit_policy: :first_hit,\n        inputs: [\n          %Tablex.Variable{\n            name: :income,\n            label: \"income\",\n            desc: nil,\n            type: :undefined,\n            path: []\n          }\n        ],\n        outputs: [\n          %Tablex.Variable{\n            name: :status,\n            label: \"status\",\n            desc: nil,\n            type: :undefined,\n            path: []\n          }\n        ],\n        rules: [\n          [1, {:input, [>: 50000]}, {:output, [\"approved\"]}],\n          [2, {:input, [<=: 49999]}, {:output, [\"declined\"]}]\n        ],\n        valid?: :undefined,\n        table_dir: :h\n      },\n      process_uid: \"b79fe224-52b9-4457-b885-1234c47c0eb3\"\n    }\n  ],\n  data: %{status: \"declined\", income: 3000},\n  complete: true\n}\n```\n\nWe see that the new property **status** with a value of **declined** has been added to the state of the completed process.","ref":"task_types.html#rule-task-example"},{"type":"extras","title":"Parallel Task - Mozart Task Types","doc":"A parallel task `Mozart.Task.Parallel` is used to create multiple concurrent process execution paths. It does this by specifying multipple next tasks in its **multi_next** field.\n\nIf you are following along, open an Elixir project that has Mozart as a dependency.\n\n```\niex -S mix\n\n```\n\nNow paste the following alias' into your iex session\n\n```\n alias Mozart.Data.ProcessModel\n alias Mozart.Task.Parallel\n alias Mozart.Task.User\n alias Mozart.ProcessEngine, as: PE\n alias Mozart.ProcessService, as: PS\n\n```\n\nNow define a process model with a parallel task and two user tasks. When the parallel task completes, the two user tasks should be opened in parallel.\n\n```\nmodel = %ProcessModel{\n        name: :parallel_process_model,\n        tasks: [\n          %Parallel{\n            name: :parallel_user_tasks,\n            multi_next: [:user_task_1, :user_task_2]\n          },\n          %User{\n            name: :user_task_1,\n            inputs: [:user_task_1_input],\n            assigned_groups: [\"admin\"]\n          },\n          %User{\n            name: :user_task_2,\n            inputs: [:user_task_2_input],\n            assigned_groups: [\"admin\"]\n          }\n        ],\n        initial_task: :parallel_user_tasks\n      }\n\n```\n\nWe specify that state data properties **:user_task_1_input** and **:user_task_2_input** will passed to **:user_task_1** and **:user_task_2**, respectively. However, in this example we aren't going to actually complete the tasks. Hence, the property values won't actually be used.\n\nNow paste the following into your iex session to execute your process model:\n\n```\nPS.load_process_model(model)\ndata = %{user_task_1_input: :input_1, user_task_2_input: :input_2}\n{:ok, ppid, uid, _process_key} = PE.start_process(:parallel_process_model, data)\nPE.execute(ppid)\n\n```\n\nNow, to view the state of the not yet complete process, paste the following into your iex session:\n\n```\nPE.get_state(ppid)\n\n```\n\nand should see something like this:\n\n```\niex [10:32 :: 11] > PE.get_state(ppid)\n%Mozart.Data.ProcessState{\n  uid: \"f7bffb0d-a66e-4381-bcba-01b9f06ac68b\",\n  parent_pid: nil,\n  model_name: :parallel_process_model,\n  start_time: ~U[2024-06-01 15:33:23.663777Z],\n  end_time: nil,\n  execute_duration: nil,\n  open_tasks: %{\n    \"0fe80867-0a8c-4721-ad57-838dbb0901ac\" => %{\n      complete: false,\n      function: nil,\n      name: :user_task_2,\n      type: :user,\n      next: nil,\n      __struct__: Mozart.Task.User,\n      uid: \"0fe80867-0a8c-4721-ad57-838dbb0901ac\",\n      assigned_groups: [\"admin\"],\n      inputs: [:user_task_2_input],\n      process_uid: \"f7bffb0d-a66e-4381-bcba-01b9f06ac68b\"\n    },\n    \"12925277-c6e0-4310-a87b-8945573017ba\" => %{\n      complete: false,\n      function: nil,\n      name: :user_task_1,\n      type: :user,\n      next: nil,\n      __struct__: Mozart.Task.User,\n      uid: \"12925277-c6e0-4310-a87b-8945573017ba\",\n      assigned_groups: [\"admin\"],\n      inputs: [:user_task_1_input],\n      process_uid: \"f7bffb0d-a66e-4381-bcba-01b9f06ac68b\"\n    }\n  },\n  completed_tasks: [\n    %{\n      function: nil,\n      name: :parallel_user_tasks,\n      type: :parallel,\n      __struct__: Mozart.Task.Parallel,\n      uid: \"aa0108cf-2f2c-4700-b97a-49ca7b002c18\",\n      multi_next: [:user_task_1, :user_task_2],\n      sub_process_model_name: nil,\n      process_uid: \"f7bffb0d-a66e-4381-bcba-01b9f06ac68b\"\n    }\n  ],\n  data: %{user_task_1_input: :input_1, user_task_2_input: :input_2},\n  complete: false\n}\n\n```\n\nWe see that parallel task **:parallel_user_tasks** has been completed. We also see our two user tasks have been opened in parallel, but have not yet been completed.","ref":"task_types.html#parallel-task"},{"type":"extras","title":"Subprocess Task - Mozart Task Types","doc":"A subprocess task (`Mozart.Task.Subprocess`) performs its work by starting a subprocess instance as specified in its **sub_process** field.\n\nIf you are following along, open an Elixir project that has Mozart as a dependency.\n\n```\niex -S mix\n\n```\n\nNow paste the following alias' into your iex session:\n\n```\n alias Mozart.Data.ProcessModel\n alias Mozart.Task.Subprocess\n alias Mozart.Task.Script\n alias Mozart.ProcessEngine, as: PE\n alias Mozart.ProcessService, as: PS\n\n```\n\nNow we need two process models. The first process model will call a subprocess task, wait for its completion, and then call a service task. The second process model will be called as a subprocess of the first process model.\n\n```\nmodels = \n    [\n      %ProcessModel{\n        name: :call_process_model,\n        tasks: [\n          %Subprocess{\n            name: :call_process_task,\n            sub_process_model_name: :service_subprocess_model,\n            next: :service_task1\n          },\n          %Script{\n            name: :service_task1,\n            function: fn data -> Map.put(data, :value, data.value + 1) end\n          }\n        ],\n        initial_task: :call_process_task\n      },\n      %ProcessModel{\n        name: :service_subprocess_model,\n        tasks: [\n          %Script{\n            name: :service_task,\n            function: fn data -> Map.put(data, :subprocess_data, \"subprocess data\") end\n          }\n        ],\n        initial_task: :service_task\n      }\n    ]\n\n```\n\nNow paste the following into your iex session to execute your process model:\n\n```\nPS.clear_state()\nPS.load_process_models(models)\ndata = %{value: 1}\n{:ok, ppid, uid, _process_key} = PE.start_process(:call_process_model, data)\nPE.execute(ppid)\n\n```\n\nAt this point, both the processess should have completed.\n\n```\nPS.get_completed_processes()\n\n```\n\nand should see something like this:\n\n```\niex [12:30 :: 12] > PS.get_completed_processes()\n[\n  %Mozart.Data.ProcessState{\n    uid: \"0a04f109-505b-4059-a5e2-0980c3229ce2\",\n    parent_pid: #PID<0.299.0>,\n    model_name: :service_subprocess_model,\n    start_time: ~U[2024-06-02 17:30:33.946285Z],\n    end_time: ~U[2024-06-02 17:30:33.946645Z],\n    execute_duration: 360,\n    open_tasks: %{},\n    completed_tasks: [\n      %{\n        function: #Function<42.105768164/1 in :erl_eval.expr/6>,\n        name: :service_task,\n        type: :service,\n        next: nil,\n        __struct__: Mozart.Task.Script,\n        uid: \"c4466a99-6f07-465c-802a-2b8836499940\",\n        inputs: nil,\n        process_uid: \"0a04f109-505b-4059-a5e2-0980c3229ce2\"\n      }\n    ],\n    data: %{value: 1, subprocess_data: \"subprocess data\"},\n    complete: true\n  },\n  %Mozart.Data.ProcessState{\n    uid: \"83769669-388d-490a-a326-cd5c3c684361\",\n    parent_pid: nil,\n    model_name: :call_process_model,\n    start_time: ~U[2024-06-02 17:30:33.939824Z],\n    end_time: ~U[2024-06-02 17:30:33.947097Z],\n    execute_duration: 7273,\n    open_tasks: %{},\n    completed_tasks: [\n      %{\n        function: #Function<42.105768164/1 in :erl_eval.expr/6>,\n        name: :service_task1,\n        type: :service,\n        next: nil,\n        __struct__: Mozart.Task.Script,\n        uid: \"692c0af0-b5f5-4474-a478-6de3cdd84ec8\",\n        inputs: nil,\n        process_uid: \"83769669-388d-490a-a326-cd5c3c684361\"\n      },\n      %{\n        complete: true,\n        data: %{},\n        function: nil,\n        name: :call_process_task,\n        type: :sub_process,\n        next: :service_task1,\n        __struct__: Mozart.Task.Subprocess,\n        uid: \"ee4211a0-ee8e-4b6d-992d-80121566bd41\",\n        sub_process_model_name: :service_subprocess_model,\n        process_uid: \"83769669-388d-490a-a326-cd5c3c684361\",\n        completed_sub_tasks: []\n      }\n    ],\n    data: %{value: 2, subprocess_data: \"subprocess data\"},\n    complete: true\n  }\n]\n```\n\nWe see that two processes have completed. The process named **:call_process_model** is the top level process, and the named **:service_subprocess_model** is the subprocess. As expected, the top level process state shows two completed tasks and the subprocess state shows one completed task.","ref":"task_types.html#subprocess-task"},{"type":"extras","title":"Timer Task - Mozart Task Types","doc":"A **timer task** (`Mozart.Task.Timer`) performs its work by delaying a process execution path by the duration specified in its **timer_duration** field.\n\nIf you are following along, open an Elixir project that has Mozart as a dependency.\n\n```\niex -S mix\n\n```\n\nNow paste the following alias' into your iex session:\n\n```\n alias Mozart.Data.ProcessModel\n alias Mozart.Task.Timer\n alias Mozart.ProcessEngine, as: PE\n alias Mozart.ProcessService, as: PS\n\n```\n\nNow we need two process models. The first process model will call a subprocess task, wait for its completion, and then call a service task. The second process model will be called as a subprocess of the first process model.\n\n```\nmodel = \n    %ProcessModel{\n        name: :call_timer_task,\n        tasks: [\n          %Timer{\n            name: :wait_5_seconds,\n            timer_duration: 5000\n          },\n        ],\n        initial_task: :wait_5_seconds\n      }\n\n```\n\nNow paste the following into your iex session to execute your process model:\n\n```\nPS.clear_state()\nPS.load_process_model(model)\ndata = %{}\n{:ok, ppid, uid, _process_key} = PE.start_process(:call_timer_task, data)\nPE.execute(ppid)\n\n```\n\nThe process should take about 5 seconds to complete. Watch for logging to report **process complete**. After that, invoke the following:\n\n```\nPS.get_completed_processes()\n\n```\n\nand should see something like this:\n\n```\n[\n  %Mozart.Data.ProcessState{\n    uid: \"67e7f64e-7a06-4622-9b9e-1d89b552ae3b\",\n    parent_pid: nil,\n    model_name: :call_timer_task,\n    start_time: ~U[2024-06-02 18:07:18.670993Z],\n    end_time: ~U[2024-06-02 18:07:23.677426Z],\n    execute_duration: 5006433,\n    open_tasks: %{},\n    completed_tasks: [\n      %{\n        function: nil,\n        name: :wait_5_seconds,\n        type: :timer,\n        next: nil,\n        expired: true,\n        __struct__: Mozart.Task.Timer,\n        uid: \"eacaaf01-37b9-43ef-be81-d2ccb8802084\",\n        timer_duration: 5000,\n        process_uid: \"67e7f64e-7a06-4622-9b9e-1d89b552ae3b\"\n      }\n    ],\n    data: %{},\n    complete: true\n  }\n]\n\n```\n\nNotice in the above that **execution_duration** is 5006433 microseconds, i.e. just a little over 5 seconds.","ref":"task_types.html#timer-task"},{"type":"extras","title":"Join Task - Mozart Task Types","doc":"The **join task** (`Mozart.Task.Join`) is used to sychronize any number of parallel execution paths. The **inputs** field holds a list of task names that must finish before the join task can be completed.\n\n\nIf you are following along, open an Elixir project that has Mozart as a dependency.\n\n```\niex -S mix\n\n```\n\nNow paste the following alias' into your iex session:\n\n```\n alias Mozart.Data.ProcessModel\n alias Mozart.Task.Join\n alias Mozart.Task.Parallel\n alias Mozart.Task.Script\n alias Mozart.Task.Timer\n alias Mozart.ProcessEngine, as: PE\n alias Mozart.ProcessService, as: PS\n\n```\n\nThe process model below has a parallel task that spawns tasks **:foo** and **:bar** in parallel.\n\n```\nmodel = \n    %ProcessModel{\n        name: :parallel_process_model,\n        tasks: [\n          %Parallel{\n            name: :parallel_task,\n            multi_next: [:service_task_1, :timer_task]\n          },\n          %Script{\n            name: :service_task_1,\n            function: fn data -> Map.merge(data, %{foo: :foo}) end,\n            next: :join_task\n          },\n          %Timer{\n            name: :timer_task,\n            timer_duration: 10000,\n            next: :service_task_after_timer\n          },\n          %Script{\n            name: :service_task_after_timer,\n            function: fn data -> Map.merge(data, %{foo_bar: :foo_bar}) end,\n            next: :join_task\n          },\n          %Join{\n            name: :join_task,\n            inputs: [:service_task_1, :service_task_after_timer],\n            next: :final_service\n          },\n          %Script{\n            name: :final_service,\n            function: fn data -> Map.merge(data, %{final: :final}) end\n          }\n        ],\n        initial_task: :parallel_task\n    }\n\n```\n\nNow paste the following into your iex session to execute your process model:\n\n```\nPS.clear_state()\nPS.load_process_model(model)\ndata = %{}\n{:ok, ppid, uid, _process_key} = PE.start_process(:parallel_process_model, data)\nPE.execute(ppid)\n\n```\n\nDue to the presense of the timer task, the process should take about 10 seconds to complete. Watch for logging to report **process complete**. After that, invoke the following:\n\n```\nPS.get_completed_processes()\n\n```\n\nand should see something like this:\n\n```\n[\n  %Mozart.Data.ProcessState{\n    uid: \"5e2e36c7-f43e-4586-8819-6ec046c4073a\",\n    parent_pid: nil,\n    model_name: :parallel_process_model,\n    start_time: ~U[2024-06-03 15:02:40.056941Z],\n    end_time: ~U[2024-06-03 15:02:50.065079Z],\n    execute_duration: 10008138,\n    open_tasks: %{},\n    completed_tasks: [\n      %{\n        function: #Function<42.105768164/1 in :erl_eval.expr/6>,\n        name: :final_service,\n        type: :service,\n        next: nil,\n        __struct__: Mozart.Task.Script,\n        uid: \"3c64e2ea-7e42-47dd-b606-2fb61b505b7b\",\n        inputs: nil,\n        process_uid: \"5e2e36c7-f43e-4586-8819-6ec046c4073a\"\n      },\n      %{\n        name: :join_task,\n        type: :join,\n        next: :final_service,\n        __struct__: Mozart.Task.Join,\n        uid: \"e4545b2c-2d6a-4b7a-9bd7-a08866aa0e4c\",\n        inputs: [],\n        process_uid: \"5e2e36c7-f43e-4586-8819-6ec046c4073a\"\n      },\n      %{\n        function: #Function<42.105768164/1 in :erl_eval.expr/6>,\n        name: :service_task_after_timer,\n        type: :service,\n        next: :join_task,\n        __struct__: Mozart.Task.Script,\n        uid: \"28c9aef1-0e69-4a7f-a3b7-549e582dfe24\",\n        inputs: nil,\n        process_uid: \"5e2e36c7-f43e-4586-8819-6ec046c4073a\"\n      },\n      %{\n        function: nil,\n        name: :timer_task,\n        type: :timer,\n        next: :service_task_after_timer,\n        expired: true,\n        __struct__: Mozart.Task.Timer,\n        uid: \"cac585aa-b858-4d21-b9f2-38324fc4170b\",\n        timer_duration: 10000,\n        process_uid: \"5e2e36c7-f43e-4586-8819-6ec046c4073a\"\n      },\n      %{\n        function: #Function<42.105768164/1 in :erl_eval.expr/6>,\n        name: :service_task_1,\n        type: :service,\n        next: :join_task,\n        __struct__: Mozart.Task.Script,\n        uid: \"d9e8f3eb-5d56-4772-94a3-e5d6243277e3\",\n        inputs: nil,\n        process_uid: \"5e2e36c7-f43e-4586-8819-6ec046c4073a\"\n      },\n      %{\n        name: :parallel_task,\n        type: :parallel,\n        __struct__: Mozart.Task.Parallel,\n        uid: \"9a5bca86-8de4-4dc0-8477-47dc97cdbd7d\",\n        multi_next: [:service_task_1, :timer_task],\n        process_uid: \"5e2e36c7-f43e-4586-8819-6ec046c4073a\"\n      }\n    ],\n    data: %{final: :final, foo: :foo, foo_bar: :foo_bar},\n    complete: true\n  }\n]\n\n```\n\nNotice in the above that **execution_duration** is 10008138 microseconds, i.e. just a little over 10 seconds.","ref":"task_types.html#join-task"},{"type":"extras","title":"Send and Receive Tasks - Mozart Task Types","doc":"In this section we will demo both the **send** and **receive** tasks since they will normally be used together.\n\nThe **receive task** (`Mozart.Task.Receive`) waits until a message has been received from a **send task** (`Mozart.Task.Send`). The send and receive tasks can reside in either the same of different process instances.\n\n\nIf you are following along, open an Elixir project that has Mozart as a dependency.\n\n```\niex -S mix\n\n```\n\nNow paste the following alias' into your iex session:\n\n```\n alias Mozart.Data.ProcessModel\n alias Mozart.Task.Send\n alias Mozart.Task.Receive\n alias Mozart.ProcessEngine, as: PE\n alias Mozart.ProcessService, as: PS\n\n```\n\nThe process models below contain one receiving process and one sending process.\n\n```\nmodels = \n    [\n      %ProcessModel{\n        name: :process_with_receive_task,\n        tasks: [\n          %Receive{\n            name: :receive_task,\n            message_selector: fn msg ->\n              case msg do\n                :message -> %{message: true}\n                _ -> false\n              end\n            end\n          }\n        ],\n        initial_task: :receive_task\n      },\n      %ProcessModel{\n        name: :process_with_single_send_task,\n        tasks: [\n          %Send{\n            name: :send_task,\n            message: :message\n          }\n        ],\n        initial_task: :send_task\n      }\n    ]\n\n```\n\nNow paste the following into your iex session to execute, first, the receive process, and then the send process instance. We insert a 5 second delay between the two processes.\n\n```\nPS.clear_state()\nPS.load_process_models(models)\ndata = %{}\n\n{:ok, r_ppid, r_uid, _process_key} = PE.start_process(:process_with_receive_task, data)\nPE.execute(r_ppid)\nProcess.sleep(5000)\n\n{:ok, s_ppid, s_uid, _process_key} = PE.start_process(:process_with_single_send_task, data)\nPE.execute(s_ppid)\n\n```\n\nDue to the presense of the timer task, the process should take about 10 seconds to complete. Watch for logging to report **process complete**. After that, invoke the following:\n\n```\nPS.get_completed_processes()\n\n```\n\nand should see something like this:\n\n```\n[\n  %Mozart.Data.ProcessState{\n    uid: \"29e8f3b7-21df-4e47-b689-9e3fd82cf34a\",\n    parent_pid: nil,\n    model_name: :process_with_single_send_task,\n    start_time: ~U[2024-06-03 15:30:18.247638Z],\n    end_time: ~U[2024-06-03 15:30:18.248965Z],\n    execute_duration: 1327,\n    open_tasks: %{},\n    completed_tasks: [\n      %{\n        message: :message,\n        name: :send_task,\n        type: :send,\n        next: nil,\n        __struct__: Mozart.Task.Send,\n        uid: \"7b3e4c42-d68d-4d96-8d40-72f0de8d84e1\",\n        process_uid: \"29e8f3b7-21df-4e47-b689-9e3fd82cf34a\"\n      }\n    ],\n    data: %{},\n    complete: true\n  },\n  %Mozart.Data.ProcessState{\n    uid: \"3dfad9f6-2629-4e73-8ec2-c6209e84aab9\",\n    parent_pid: nil,\n    model_name: :process_with_receive_task,\n    start_time: ~U[2024-06-03 15:30:13.241838Z],\n    end_time: ~U[2024-06-03 15:30:18.248982Z],\n    execute_duration: 5007144,\n    open_tasks: %{},\n    completed_tasks: [\n      %{\n        complete: true,\n        data: %{message: true},\n        function: nil,\n        name: :receive_task,\n        type: :receive,\n        next: nil,\n        __struct__: Mozart.Task.Receive,\n        uid: \"d43b54db-2f72-4fcd-bbdc-f5c66cd330a3\",\n        message_selector: #Function<42.105768164/1 in :erl_eval.expr/6>,\n        process_uid: \"3dfad9f6-2629-4e73-8ec2-c6209e84aab9\"\n      }\n    ],\n    data: %{message: true},\n    complete: true\n  }\n]\n\n```\n\nNotice in the above that **execution_duration** for the receive process is 5007144 microseconds, i.e. just a little over the 5 second process sleep pause.","ref":"task_types.html#send-and-receive-tasks"},{"type":"extras","title":"Home Loan Example","doc":"# Home Loan Example\n\nIn this guide we will construct and execute a somewhat more complex process model than we have dealt with thus far. We will mainly use **user tasks** because they allow use to easily control of the process execution path during process exploration.\n\nWe will create a process model applying for a home loan from a bank. It will be highly simplified compared to the actual process used by lending institutions, but complicated enough for our purposes here.\n\nHere is a summary of the loan approval process:\n\n1. Get pre-approval from the bank for a loan. They will require a credit score, yearly income and the extent of the applicant's indebtedness.\n1. After the applicant secures a contract on a house, he will fill out a complete loan application. Part of that will be the purchase price of the house.\n1. Now the bank will process the applicant's loan application. This is to ensure that everything is in order and they have everything that they need.\n1. After the loan has been processed, it will go to the underwriting department to determine whether the loan will be approved or not.\n1. Finally, after the loan is approced, the bank will notify the applicant that the loan is approved.\n\nIf you are following along, open an Elixir project that has Mozart as a dependency.\n\n```\niex -S mix\n\n```\n\nNow paste the following alias' into your iex session\n\n```\n  alias Mozart.ProcessService, as: PS\n  alias Mozart.ProcessEngine, as: PE\n  alias Mozart.Data.ProcessModel\n  alias Mozart.Task.User\n  alias Mozart.Task.Case\n\n```\n\nThe following is our loan processing process model. At this point, you might try to map the process model to the summary of the process given above.\n\nWhen you are ready, paste the process model into your iex session:\n\n```\nmodel = \n    %ProcessModel{\n      name: :home_loan_process,\n      tasks: [\n        %User{\n          name: :perform_pre_approval,\n          inputs: [:credit_score, :income, :debt_amount],\n          assigned_groups: [\"credit\"],\n          next: :route_on_pre_approval_completion\n        },\n        %Case{\n          name: :route_on_pre_approval_completion,\n          cases: [\n            %{\n              expression: fn data -> data.pre_approval == true end,\n              next: :receive_mortgage_application\n            },\n            %{\n              expression: fn data -> data.pre_approval == false end,\n              next: :communicate_loan_denied\n            }\n          ]\n        },\n        %User{\n          name: :receive_mortgage_application,\n          inputs: [:credit_score, :income, :debt_amount],\n          assigned_groups: [\"credit\"],\n          next: :process_loan\n        },\n        %User{\n          name: :process_loan,\n          inputs: [:purchase_price, :credit_score, :income, :debt_amount],\n          assigned_groups: [\"credit\"],\n          next: :process_loan_outcome\n        },\n        %Case{\n          name: :process_loan_outcome,\n          cases: [\n            %{\n              expression: fn data -> data.loan_verified == true end,\n              next: :perform_underwriting\n            },\n            %{\n              expression: fn data -> data.loan_verified == false end,\n              next: :communicate_loan_denied\n            }\n          ]\n        },\n        %User{\n          name: :perform_underwriting,\n          inputs: [:purchase_price, :credit_score, :income, :debt_amount, :loan_verified],\n          assigned_groups: [\"underwriting\"],\n          next: :route_from_underwriting\n        },\n        %Case{\n          name: :route_from_underwriting,\n          cases: [\n            %{\n              expression: fn data -> data.loan_approved == true end,\n              next: :communicate_approval\n            },\n            %{\n              expression: fn data -> data.loan_approved == false end,\n              next: :communicate_loan_denied\n\n            }\n          ]\n        },\n        %User{\n          name: :communicate_approval,\n          inputs: [:loan_approved],\n          assigned_groups: [\"credit\"]\n        },\n        %User{\n          name: :communicate_loan_denied,\n          inputs: [:loan_approved],\n          assigned_groups: [\"credit\"]\n        },\n      ],\n      initial_task: :perform_pre_approval\n      }\n\n```\n\nNow let's start executing this process. Paste the following into your iex session:\n\n```\nPS.clear_state()\nPS.load_process_model(model)\ndata = %{credit_score: 700, income: 100_000, debt_amount: 20_000}\n{:ok, ppid, _uid, _process_key} = PE.start_process(:home_loan_process, data)\nPE.execute(ppid)\n\n```\n\nAt this point, the user task to perform pre-approval should have been opened:\n\n```\n[user_task] = PS.get_user_tasks_for_groups([\"credit\"])\n\n```\n\nwhich returns:\n\n```\niex [11:54 :: 13] > [user_task] = PS.get_user_tasks_for_groups([\"credit\"])\n[\n  %{\n    complete: false,\n    data: %{credit_score: 700, income: 100000, debt_amount: 20000},\n    function: nil,\n    name: :perform_pre_approval,\n    type: :user,\n    next: :route_on_pre_approval_completion,\n    __struct__: Mozart.Task.User,\n    uid: \"a987e7f0-15ba-422d-a26d-7ff6a6bdaaad\",\n    assigned_groups: [\"credit\"],\n    inputs: [:credit_score, :income, :debt_amount],\n    process_uid: \"5ece8627-623b-46f1-afa2-0fb11659a874\"\n  }\n]\n```\n\nLet's assume that is sufficient information for granting pre-approval. We can do this like so:\n\n```\n PS.complete_user_task(ppid, user_task.uid, %{pre_approval: true})\n\n```\n\nAt this point, the case task should have completed and routed the process to the user task **receive_mortgage_application**. The task will be completed when the loan applicant provides the bank with an mortgage application for a house that has been selected for purchase. The bank user will complete this task by entering the purchase price of the house.\n\nLet's do that now:\n\n```\n[user_task] = PS.get_user_tasks_for_groups([\"credit\"])\nPS.complete_user_task(ppid, user_task.uid, %{purchase_price: 500_000})\n\n```\n\nWe have now accepted the customers loan application and noted that the purchase price is $500,000. We are ready to process the loan. Loan processsing is the bank's procedure to ensure that all of the information gathers thus far is in order. A user task for processing the loan should now be open. Let's check that it is:\n\n```\n[user_task] = PS.get_user_tasks_for_groups([\"credit\"])\n\n```\n\nwhich produces:\n\n```\niex [18:16 :: 17] > [user_task] = PS.get_user_tasks_for_groups([\"credit\"])\n[\n  %{\n    complete: false,\n    data: %{\n      credit_score: 700,\n      income: 100000,\n      debt_amount: 20000,\n      purchase_price: 500000\n    },\n    function: nil,\n    name: :process_loan,\n    type: :user,\n    next: :process_loan_outcome,\n    __struct__: Mozart.Task.User,\n    uid: \"2aa15c30-b147-469d-b8a5-34946b2165df\",\n    assigned_groups: [\"credit\"],\n    inputs: [:purchase_price, :credit_score, :income, :debt_amount],\n    process_uid: \"8d106b1c-32f4-43b6-a168-e663bb59056f\"\n  }\n]\n```\n\nNow let's complete that task, asserting that the loan has been verified:\n\n```\nPS.complete_user_task(ppid, user_task.uid, %{loan_verified: true})\n\n```\n\nNow we are ready for the underwriting department to assess whether the loan should be approved. Let's query for the task and approve the loan:\n\n```\n[user_task] = PS.get_user_tasks_for_groups([\"underwriting\"])\nPS.complete_user_task(ppid, user_task.uid, %{loan_approved: true})\n\n```\n\nWe are now ready for the final task of our process - informing the customer that the loan has been approved. Let's do that now:\n\n```\n[user_task] = PS.get_user_tasks_for_groups([\"credit\"])\nPS.complete_user_task(ppid, user_task.uid, %{loan_approved: true})\n\n```\n\nAt this point, our logs should indicate that the process has finished, as indeed we do:\n\n```\niex [18:16 :: 23] > PS.complete_user_task(ppid, user_task.uid, %{loan_approved: true})\n:ok\n18:46:03.183 [info] Complete user task [communicate_approval][53a2b9ae-ba89-49f2-beba-0fdd8f49ab1f]\n18:46:03.183 [info] Process complete [home_loan_process][8d106b1c-32f4-43b6-a168-e663bb59056f]\n```\n\nNow let's start over at the beginning, and enter data so that the process completes with the loan being declined:\n\n```\nPS.clear_state()\nPS.load_process_model(model)\ndata = %{credit_score: 700, income: 100_000, debt_amount: 20_000}\n{:ok, ppid, _uid, _process_key} = PE.start_process(:home_loan_process, data)\nPE.execute(ppid)\n\n```\n\nNow, instead of granting pre-approval, let's decline the loan with the following:\n\n```\n[user_task] = PS.get_user_tasks_for_groups([\"credit\"])\nPS.complete_user_task(ppid, user_task.uid, %{pre_approval: false})\n\n```\n\nWe see in the last log entry, that we have a new **communicate_loan_denied** task.\n\n```\niex [18:16 :: 32] > PS.complete_user_task(ppid, user_task.uid, %{pre_approval: false})\n:ok\n18:53:57.138 [info] New task instance [route_on_pre_approval_completion][645b421a-11ef-461e-91a7-7aa7e7fc66e6]\n18:53:57.138 [info] Complete user task [perform_pre_approval][3ee4b0d2-1432-4d54-b248-34ae928db983]\n18:53:57.138 [info] Complete case task [route_on_pre_approval_completion][645b421a-11ef-461e-91a7-7aa7e7fc66e6]\n18:53:57.139 [info] New task instance [communicate_loan_denied][a6a5bacd-a5a4-4b1e-a39b-99c0c8b58d04]\n```\n\nLet's complete that task now:\n\n```\n[user_task] = PS.get_user_tasks_for_groups([\"credit\"])\nPS.complete_user_task(ppid, user_task.uid, %{loan_approved: false})\n\n```","ref":"home_loan.html"},{"type":"extras","title":"Mozart Event Types","doc":"# Mozart Event Types\n\nCurrently, there is just one event type as described below. This document will be updated as addtional event types are implemented.","ref":"event_types.html"},{"type":"extras","title":"Task Exit Event - Mozart Event Types","doc":"A **Task Exit Event** (`Mozart.Event.TaskExit`) allows a process execution to exit an open task in response to an external event (a Phoenix.PubSub event). When the task exits in this way, process execution will follow an alternate execution path as specified the the *task exit event*.\n\nThe *task exit event* is implemented by a struct with the following fields:\n\n* A **name** field with uniquely identifies the event within the scope of the containing process model.\n* Optionally, a **function** field can be used to alter the process data when the event is handled.\n* **message_selector** field requires a function specification used to select Phoenix.PubSub events.\n* The **exit_task** field specifies the name of the task that would exit in response to the event.\n* The **next** field specifies the name of the next task, if any, that should be opened in response to the event.\n* Finally, a **type** field that will default to  **:task_exit** for this event type.","ref":"event_types.html#task-exit-event"},{"type":"extras","title":"Task Exit Event Example - Mozart Event Types","doc":"If you are following along, open an Elixir project that has Mozart as a dependency.\n\n```\niex -S mix\n\n```\n\nNow paste the following alias' into your iex session\n\n```\n alias Mozart.Data.ProcessModel\n alias Mozart.ProcessEngine, as: PE\n alias Mozart.ProcessService, as: PS\n alias Mozart.Task.Script\n alias Mozart.Task.User\n alias Mozart.Task.Subprocess\n alias Mozart.Event.TaskExit\n alias Phoenix.PubSub\n\n```\n\nNow we define two process models - one top level model and one subprocess model. We specify a **TaskExit** event whose occurence will cause the subprocess to exit and a new service task to be opened.\n\nGo ahead and paste it into your iex session:\n\n```\nmodels = [\n      %ProcessModel{\n        name: :simple_call_process_model,\n        tasks: [\n          %Subprocess{\n            name: :call_process_task,\n            sub_process_model_name: :sub_process_with_one_user_task\n          },\n          %Script{\n            name: :service_after_task_exit,\n            function: fn data -> Map.put(data, :service_after_task_exit, true) end\n          }\n        ],\n        events: [\n          %TaskExit{\n            name: :exit_sub_process,\n            exit_task: :call_process_task,\n            message_selector: fn msg ->\n              case msg do\n                :exit_user_task -> true\n                _ -> nil\n              end\n            end,\n            next: :service_after_task_exit\n          }\n        ],\n        initial_task: :call_process_task\n      },\n    %ProcessModel{\n      name: :sub_process_with_one_user_task,\n      tasks: [\n        %User{\n          name: :user_task,\n          assigned_groups: [\"admin\"]\n        }\n      ],\n      initial_task: :user_task\n    }\n  ]\n\n```\n\nNow let's load the process models, and then start and execute the process top level process model:\n\n```\nPS.clear_state()\nPS.load_process_models(models)\ndata = %{}\n\n{:ok, ppid, _uid, process_key} = PE.start_process(:simple_call_process_model, data)\nPE.execute(ppid)\n\n```\n\nand we should see in our logs:\n\n```\n15:59:17.305 [info] Start process instance [simple_call_process_model][3a205a5b-07a3-4ecc-926e-0f405eddd0ac]\n{:ok, #PID<0.285.0>, \"3a205a5b-07a3-4ecc-926e-0f405eddd0ac\",\n \"272c2c9c-adcb-444d-a010-479aa68e6025\"}\niex [15:58 :: 14] > PE.execute(ppid)\n:ok\n15:59:17.308 [info] New task instance [call_process_task][7404314e-3590-459a-a71f-458a5416966a]\n15:59:17.308 [info] Start process instance [sub_process_with_one_user_task][cf66c2a2-75e9-4649-b4bd-070ca9e3746a]\n15:59:17.309 [info] New task instance [user_task][106e85b5-082a-47b9-ab66-4606617efc7e]\n```\n\nNotice that two processes have been started - the top level process we explicitly started, and a subprocess. Let's use the process key to examine the state of both processes:\n\n```\nPS.get_processes_for_process_key(process_key)\n\n```\n\nWe should see something like this:\n\n```\niex [15:58 :: 15] > PS.get_processes_for_process_key(process_key)\n[\n  %Mozart.Data.ProcessState{\n    uid: \"3a205a5b-07a3-4ecc-926e-0f405eddd0ac\",\n    process_key: \"272c2c9c-adcb-444d-a010-479aa68e6025\",\n    parent_pid: nil,\n    model_name: :simple_call_process_model,\n    start_time: ~U[2024-06-12 20:59:17.302851Z],\n    end_time: nil,\n    execute_duration: nil,\n    open_tasks: %{\n      \"7404314e-3590-459a-a71f-458a5416966a\" => %{\n        complete: false,\n        data: %{},\n        name: :call_process_task,\n        type: :sub_process,\n        next: nil,\n        __struct__: Mozart.Task.Subprocess,\n        uid: \"7404314e-3590-459a-a71f-458a5416966a\",\n        sub_process_model_name: :sub_process_with_one_user_task,\n        sub_process_pid: #PID<0.286.0>,\n        start_time: ~U[2024-06-12 20:59:17.308810Z],\n        finish_time: nil,\n        duration: nil,\n        process_uid: \"3a205a5b-07a3-4ecc-926e-0f405eddd0ac\"\n      }\n    },\n    completed_tasks: [],\n    data: %{},\n    complete: false\n  },\n  %Mozart.Data.ProcessState{\n    uid: \"cf66c2a2-75e9-4649-b4bd-070ca9e3746a\",\n    process_key: \"272c2c9c-adcb-444d-a010-479aa68e6025\",\n    parent_pid: #PID<0.285.0>,\n    model_name: :sub_process_with_one_user_task,\n    start_time: ~U[2024-06-12 20:59:17.308880Z],\n    end_time: nil,\n    execute_duration: nil,\n    open_tasks: %{\n      \"106e85b5-082a-47b9-ab66-4606617efc7e\" => %{\n        complete: false,\n        function: nil,\n        name: :user_task,\n        type: :user,\n        next: nil,\n        __struct__: Mozart.Task.User,\n        uid: \"106e85b5-082a-47b9-ab66-4606617efc7e\",\n        assigned_groups: [\"admin\"],\n        start_time: ~U[2024-06-12 20:59:17.309097Z],\n        finish_time: nil,\n        duration: nil,\n        inputs: nil,\n        process_uid: \"cf66c2a2-75e9-4649-b4bd-070ca9e3746a\"\n      }\n    },\n    completed_tasks: [],\n    data: %{},\n    complete: false\n  }\n]\n\n```\n\nAs we expected, we see that there are two processes, each having one open task. The high level process has an open subprocess task. The subprocess has a user task open.\n\nNow, we want to generate a task exit event on the open task in the high level process. In practice, this might be done using a send task in another process engine instance. For the purpose of our example, we will just broadcast a PubSub event as follows:\n\n```\n PubSub.broadcast(:pubsub, \"pe_topic\", {:event, :exit_user_task})\n\n```\n\nThis should result in the following output:\n\n```\niex [19:12 :: 17] > PubSub.broadcast(:pubsub, \"pe_topic\", {:event, :exit_user_task})\n:ok\n19:14:04.732 [info] Process complete due to task exit event [sub_process_with_one_user_task][97f945f1-b508-44ec-87ad-812d7dbb1a3d]\n19:14:04.732 [info] New task instance [service_after_task_exit][20e12b5f-dbe9-4d17-b008-57342963ac45]\n19:14:04.732 [info] Complete service task [service_after_task_exit[20e12b5f-dbe9-4d17-b008-57342963ac45]\n19:14:04.732 [info] Process complete [simple_call_process_model][059eccc4-a0f6-4f1a-8009-6fcf7b622e87]\n```\n\n* The first log statement tells us that the subprocess exited due to a task exit event.\n* The second log statement shows that a new service task was opened and it is the task specified in the **next** field of the **TaskExit** event.\n* The third log statement shows that the service task completed.\n* And, finally, the last log statement shows that out top level process completed.","ref":"event_types.html#task-exit-event-example"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,51,46,48]}}